import { Injectable } from '@angular/core';
import { File, Transfer } from 'ionic-native';
import { ImageLoaderConfig } from "./image-loader-config";
export var ImageLoader = (function () {
    function ImageLoader(config) {
        var _this = this;
        this.config = config;
        this.isCacheReady = false;
        this.isInit = false;
        if (window.location.protocol === 'http:' || window.location.protocol === 'https:') {
            // we are running on a browser, or using livereload
            // plugin will not function in this case
            this.isInit = true;
            this.throwWarning('You are running on a browser or using livereload, IonicImageLoader will not function, falling back to browser loading.');
        }
        else {
            document.addEventListener('deviceready', function () {
                _this.initCache();
            }, false);
        }
    }
    ImageLoader.prototype.downloadImage = function (imageUrl, localPath) {
        var transfer = new Transfer();
        return transfer.download(imageUrl, localPath);
    };
    ImageLoader.prototype.getImagePath = function (imageUrl) {
        var _this = this;
        return new Promise(function (resolve) {
            var getImage = function () {
                _this.getCachedImagePath(imageUrl)
                    .then(function (imagePath) {
                    resolve(imagePath);
                })
                    .catch(function () {
                    // image doesn't exist in cache, lets fetch it and save it
                    var localPath = cordova.file.cacheDirectory + _this.config.cacheDirectoryName + '/' + _this.createFileName(imageUrl);
                    _this.downloadImage(imageUrl, localPath)
                        .then(function () {
                        resolve(localPath);
                    })
                        .catch(function (e) {
                        resolve(imageUrl);
                        _this.throwError(e);
                    });
                });
            };
            var check = function () {
                if (_this.isInit) {
                    if (_this.isCacheReady) {
                        getImage();
                    }
                    else {
                        _this.throwWarning('The cache system is not running. Images will be loaded by your browser instead.');
                        resolve(imageUrl);
                    }
                }
                else {
                    setTimeout(function () { return check(); }, 250);
                }
            };
            check();
        });
    };
    ImageLoader.prototype.initCache = function (replace) {
        var _this = this;
        if (!this.filePluginExists) {
            this.isInit = true;
            return;
        }
        this.cacheDirectoryExists
            .then(function (exists) {
            _this.isCacheReady = true;
            _this.isInit = true;
        })
            .catch(function (e) {
            _this.createCacheDirectory(replace)
                .then(function () {
                _this.isCacheReady = true;
                _this.isInit = true;
            })
                .catch(function (e) {
                _this.throwError(e);
                _this.isInit = true;
            });
        });
    };
    ImageLoader.prototype.getCachedImagePath = function (url) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (!_this.isCacheReady) {
                return reject();
            }
            var fileName = _this.createFileName(url);
            var dirPath = cordova.file.cacheDirectory + _this.config.cacheDirectoryName;
            File.resolveLocalFilesystemUrl(dirPath + '/' + fileName)
                .then(function (fileEntry) {
                resolve(fileEntry.nativeURL);
            })
                .catch(reject);
        });
    };
    ImageLoader.prototype.throwError = function (error) {
        if (this.config.debugMode) {
            console.error('ImageLoader Error', error);
        }
    };
    ImageLoader.prototype.throwWarning = function (error) {
        if (this.config.debugMode) {
            console.warn('ImageLoader Warning', error);
        }
    };
    Object.defineProperty(ImageLoader.prototype, "filePluginExists", {
        get: function () {
            if (!cordova || !cordova.file) {
                this.throwWarning('Unable to find the cordova file plugin. ImageLoader will not cache images.');
                return false;
            }
            return true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ImageLoader.prototype, "cacheDirectoryExists", {
        get: function () {
            return File.checkDir(cordova.file.cacheDirectory, this.config.cacheDirectoryName);
        },
        enumerable: true,
        configurable: true
    });
    ImageLoader.prototype.createCacheDirectory = function (replace) {
        if (replace === void 0) { replace = false; }
        return File.createDir(cordova.file.cacheDirectory, this.config.cacheDirectoryName, replace);
    };
    /**
     * Creates a unique file name out of the URL
     * @param url {string} URL of the file
     * @returns {string} Unique file name
     */
    ImageLoader.prototype.createFileName = function (url) {
        // hash the url to get a unique file name
        return this.hashString(url).toString();
    };
    /**
     * Converts a string to a unique 32-bit int
     * @param string {string} string to hash
     * @returns {number} 32-bit int
     */
    ImageLoader.prototype.hashString = function (string) {
        var hash = 0;
        var char;
        if (string.length === 0)
            return hash;
        for (var i = 0; i < string.length; i++) {
            char = string.charCodeAt(i);
            hash = ((hash << 5) - hash) + char;
            hash = hash & hash;
        }
        return hash;
    };
    ImageLoader.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    ImageLoader.ctorParameters = [
        { type: ImageLoaderConfig, },
    ];
    return ImageLoader;
}());
//# sourceMappingURL=image-loader.js.map